#!/usr/bin/python
#
# GUI for Update Agent
# Copyright (c) 1999-2001 Red Hat, Inc.  Distributed under GPL.
#
# Author: Preston Brown <pbrown@redhat.com>

import sys, os
import time
import string

saveargs = []
for arg in sys.argv[1:]:
    saveargs.append(arg)
    sys.argv.remove(arg)

import gtk, GDK, GTK
from gtk import TRUE
from gtk import FALSE
import gnome, gnome.ui
import libglade
import signal
import rpm
import lilocfg

sys.path.append("/usr/share/rhn/up2date_client")

from translate import _, N_, cat

import progress
import advisorywin
import checklist
import up2dateErrors
import up2dateAuth
import up2date
import userpass
import config
import rhnChannel
import packageList
import gpgUtils
import rhnErrata
import up2dateUtils
from up2date import DPRINT

from wrapperUtils import wrap_text

cfg = config.initUp2dateConfig()


class GnomeTextDialog(gnome.ui.GnomeOkDialog):
    def __init__(self, text):
	    gnome.ui.GnomeOkDialog.__init__(self,"advisory Window")
	    print self.vbox
            self.vbox.set_homogeneous(1)
            self.vbox.set_spacing(1)
            self.set_usize(300,300)
            scrolled_window = gtk.GtkScrolledWindow()
            scrolled_window.set_border_width(10)
            scrolled_window.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
            self.vbox.pack_start(scrolled_window)
            scrolled_window.show()
            
	    textArea = gtk.GtkText()
            textArea.freeze()
	    textArea.insert_defaults(text)
	    textArea.thaw()
            textArea.set_editable(0)
            textArea.set_line_wrap(0)
            scrolled_window.add_with_viewport(textArea)

            textArea.show()
            
class Gui:

    def __init__(self):
        self.gotPackages = FALSE
        self.gotSkippedPackages = FALSE
        
        self.xml = libglade.GladeXML("/usr/share/rhn/up2date_client/gui.glade", "mainWin", domain="up2date")
        self.xml.signal_autoconnect (
            { "onDruidCancel" : self.onDruidCancel,
              "onStartPageNext" : self.onStartPageNext,
              "onChannelsPagePrepare" : self.onChannelsPagePrepare,
              "onChannelsPageNext" : self.onChannelsPageNext,
              "onSkippedPagePrepare" : self.onSkippedPagePrepare,
              "onSkippedPageNext" : self.onSkippedPageNext,
              "onPackagePagePrepare" : self.onPackagePagePrepare,
              "onPackagePageNext" : self.onPackagePageNext,
              "onPackagePageBack" : self.onPackagePageBack,
              "onDependencyPagePrepare" : self.onDependencyPagePrepare,
              "onRetrievalPagePrepare" : self.onRetrievalPagePrepare,
              "onRetrievalPageNext" : self.onRetrievalPageNext,
              "onRetrievalPageBack" : self.onRetrievalPageBack,
              "onInstallPagePrepare": self.onInstallPagePrepare,
              "onInstallPageBack": self.onInstallPageBack,
              "onFinishPagePrepare" : self.onFinishPagePrepare,
              "onFinishPageFinish" : self.onFinishPageFinish } )

        self.druid = self.xml.get_widget("druid")
        self.mainWin = self.xml.get_widget("mainWin")
        self.mainWin.connect("delete-event", gtk.mainquit)
        self.mainWin.connect("hide", gtk.mainquit)
        self.setupSkippedList()
        self.setupPackageList()
        self.setupChannelList()
                # some bold labels
        label = self.xml.get_widget("pkgInfoLabel1")
        style = label.get_style().copy()
        style.font = gtk.load_font("-*-helvetica-bold-r-normal--12-*-*-*-p-*-iso8859-1")
        label.set_style(style)
        label = self.xml.get_widget("pkgInfoLabel2")
        label.set_style(style)

        # background on finished page
        box = self.xml.get_widget("finishEventBox")
        style = box.get_style().copy()
        color = self.druid.get_colormap().alloc(52232, 0, 0)
        # for gtk 1.2
        style.bg[GTK.STATE_NORMAL] = color
        # for gtk 2.0
#        style.bg[gtk.STATE_NORMAL] = color
        box.set_style(style)
        
        self.mainWin.show_all()
        self.xml.get_widget("packagesFinishedLabel").set_text("...")
        self.xml.get_widget("installingFinishedLabel").set_text("...")
        

    def onDruidCancel(self, dummy):
        # FIXME: this shouldne be needed
        # this only exits the innermost mainloop
        # and I dont have a way to determine what level of mainloop
        # i'm at, so I can do a gtk.quit_add()
        
        gtk.mainquit()
        # so less be harsh and really quit...
        sys.exit()


    def fatalError(self, error):
        self.setArrowCursor()
        dlg = gnome.ui.GnomeErrorDialog(wrap_text(error), self.mainWin)
        dlg.show_all()
        dlg.run_and_close()
        
        gtk.mainquit()
        sys.exit(1)


    def setBusyCursor(self):
        # gtk 1.2
        cursor = gtk.cursor_new (GDK.WATCH)
        # gtk 2.0
        #cursor = gdk.cursor_new(gtk.gdk.WATCH)
        self.mainWin.get_window().set_cursor(cursor)
        while gtk.events_pending():
            gtk.mainiteration(FALSE)


    def setArrowCursor(self):
        # gtk 1.2
        cursor = gtk.cursor_new (GDK.LEFT_PTR)
        # gtk 2.0
        #cursor = gdk.cursor_new (gtk.gdk.LEFT_PTR)
        self.mainWin.get_window().set_cursor(cursor)
        while gtk.events_pending():
            gtk.mainiteration(FALSE)
            
        
    def updateSelectedSize(self, state, rowData):
        sizeLabel = self.xml.get_widget("sizeLabel")
         
        if state == 0:
            # weird logic, dont subtract the package size from the total
            # if we never added it...
            if not up2date.isPackageCached(rowData):
                self.totalSelectedSize = self.totalSelectedSize - int(rowData[5])
            self.pkgsSelected = self.pkgsSelected - 1
        else:
            if not up2date.isPackageCached(rowData):
                self.totalSelectedSize = self.totalSelectedSize + int(rowData[5])
            self.pkgsSelected = self.pkgsSelected + 1
        sizeLabel.set_text("%d kB" % (self.totalSelectedSize / 1024))
        if self.pkgsSelected:
            self.druid.set_buttons_sensitive(TRUE, TRUE, TRUE)
        else:
            self.druid.set_buttons_sensitive(TRUE, FALSE, TRUE)
            
    def setupSkippedList(self):
        self.currentRow = -1
        window = self.xml.get_widget("skippedWindow")
        
        self.skippedArea = checklist.CheckList(6)
        self.skippedArea.set_column_title(1, _("Package Name"))
        #self.skippedArea.set_column_min_width(1, 200)
        self.skippedArea.set_column_title(2, _("Version"))
        self.skippedArea.set_column_min_width(2, 50)
        self.skippedArea.set_column_title(3, _("Rel."))
        self.skippedArea.set_column_min_width(3, 50)
        self.skippedArea.set_column_title(4, _("Arch"))
        self.skippedArea.set_column_min_width(4, 50)
        self.skippedArea.set_column_title(5, _("Size"))
        self.skippedArea.set_column_min_width(5, 50)
        self.skippedArea.set_column_title(6, _("Reason Skipped"))
        self.skippedArea.set_column_min_width(4, 50)
        self.skippedArea.column_titles_show()
        self.skippedArea.focus_row = -1

        self.skippedArea.connect("select_row", self.showPackageInfo,
                                 (self.xml.get_widget("skippedPackageTextArea")))
        window.add(self.skippedArea)

        self.xml.get_widget("skippedSelectAllButton").connect("toggled",
                                                              self.selectAll)
        self.xml.get_widget("advisoryButton").connect("clicked",
                                                      self.showAdvisory,
                                                      self.skippedArea)

    def setupChannelList(self):
        self.currentRow = -1
        window = self.xml.get_widget("channelWindow")
        
        self.channelArea = checklist.CheckList(2)
        self.channelArea.set_column_title(1, _("Description"))
        self.channelArea.set_column_min_width(1,200)
        self.channelArea.set_column_title(2, _("Channel"))
        self.channelArea.column_titles_show()
        self.channelArea.focus_row = -1

        self.channelArea.connect("select_row", self.showChannelInfo,
                                 (self.xml.get_widget("channelInfoTextArea")))

        window.add(self.channelArea)

        
    def setupPackageList(self):
        self.currentRow = -1
        window = self.xml.get_widget("packageWindow")
        
        self.packageArea = checklist.CheckList(5)
        self.packageArea.set_column_title(1, _("Package Name"))
        #self.packageArea.set_column_max_width(1, 200)
        self.packageArea.set_column_title(2, _("Version"))
        self.packageArea.set_column_min_width(2, 50)
        self.packageArea.set_column_title(3, _("Release"))
        self.packageArea.set_column_min_width(3, 50)
        self.packageArea.set_column_title(4, _("Arch"))
        self.packageArea.set_column_min_width(4, 50)
        self.packageArea.set_column_title(5, _("Size"))
        self.packageArea.set_column_min_width(5, 50)
        self.packageArea.column_titles_show()
        self.packageArea.set_toggled_func(self.updateSelectedSize)
        self.packageArea.focus_row = -1

        self.packageArea.connect("select_row", self.showPackageInfo,
                                 (self.xml.get_widget("packageInfoTextArea")))
        window.add(self.packageArea)

        self.xml.get_widget("selectAllButton").connect("toggled",
                                                       self.selectAll)

        self.xml.get_widget("advisoryButton2").connect("clicked",
                                                       self.showAdvisory,
                                                       self.packageArea)

                                                    

    def showAdvisory(self, button, area):
        if self.currentRow == -1:
            gtk.gdk_beep()
            return

        self.setBusyCursor()

        rowData = area.get_row_data(self.currentRow)
        advisories = rhnErrata.getAdvisoryInfo(rowData[1])
        self.setArrowCursor()
        if not advisories:
            dlg = gnome.ui.GnomeOkDialog(_("No advisory information available."), self.mainWin)
            dlg.run_and_close()
            return

        for a in advisories:
            txt = "%s [%s]\n%s\n--------\n%s" % (a['errata_type'],
                                   a['advisory'],
                                   a['topic'],
                                   a['description'])
            #FIXME: make this work
#            dlg = GnomeTextDialog(wrap_text(txt))
            dlg = gnome.ui.GnomeOkDialog(wrap_text(txt), self.mainWin)
            dlg.run_and_close()           
            

    def selectAll(self, button):
        if button.get_name() == "selectAllButton":
            area = self.packageArea
        elif button.get_name() == "skippedSelectAllButton":
            area = self.skippedArea
        else:
            return
        for index in range(area.n_rows):
            rowData = area.get_row_data(index)
            if rowData[0] != button.get_active():
                area._toggle_row(index)


    def showPackageInfo(self, area, row, column, event, textarea):
        self.currentRow = row
        rowData = area.get_row_data(row)
        hdr = up2date.getHeader(rowData[1],lite=1)
        textarea.freeze()
        textarea.backward_delete(textarea.get_length())
        textarea.insert_defaults(hdr['summary'] + "\n" + hdr['description'])
        textarea.thaw()

    def showChannelInfo(self, area, row, column, event, textarea):
        # FIXME? rpc to get extended channel info?
        (row,data) = area.get_row_data(row)
        try:
            labeltext = data['description']
            if labeltext == '':
                labeltext = data['name']
        except IndexError:
            labeltext = ""
            
        textarea.freeze()
        textarea.backward_delete(textarea.get_length())
        textarea.insert_defaults(labeltext)
        textarea.thaw()

    def __refreshCallback(self):
        while gtk.events_pending():
                gtk.mainiteration(FALSE)

    def onStartPageNext(self,page,dummy):
        pass

    def updateChannelSubscriptions(self):
        # first, see if we need to subscribe/unsubscribe to any channels
        channelsToSubscribe = []
        channelsToUnsubscribe = []
        for row in range(self.channelArea.n_rows):
            row_data=self.channelArea.get_row_data(row)
            current_status = row_data[0]
            row_data_hash = row_data[1]
            # dont allow user to unsub from base channel

            if current_status != row_data_hash['initial_status']:
                if row_data_hash['parent_channel'] == "":
                    message = _("You can not unsubscribe from %s because it is a base channel") % row_data_hash['name']
                    dlg = gnome.ui.GnomeWarningDialog(wrap_text(message), self.mainWin)
                    dlg.set_modal(TRUE)
                    gtk.mainloop()
                    #FIXME: add in hook to set the sub/unsub status in the widget to subscribed
                    break
                if current_status == TRUE:
                    channelsToSubscribe.append(row_data_hash['label'])
                else:
                    channelsToUnsubscribe.append(row_data_hash['label'])
                    
        if len(channelsToSubscribe) or len(channelsToUnsubscribe):
            passwordSuccess = None
            while not passwordSuccess:
                username  = ""
                password = ""
                userpasswin = userpass.UserPass(username, password)
                userpasswin.win.set_modal(TRUE)
                if len(channelsToSubscribe):
                    ret = None
                    try:
                        ret  = rhnChannel.subscribeChannels(channelsToSubscribe,username,password)
                    except up2dateErrors.PasswordError, e:
                        passwordSuccess = None
                        continue
 
                if len(channelsToUnsubscribe):
                    ret = None
                    try:
                        ret = rhnChannel.unsubscribeChannels(channelsToUnsubscribe,username,password)
                    except up2dateErrors.PasswordError,e:
                        passwordSuccess = None
                        continue
                if ret:
                    passwordSuccess = TRUE
            # we updated the channel list, easiest thing to do
            # is probabaly to login again
            up2dateAuth.updateLoginInfo()

    def onChannelsPageNext(self, page, dummy):

        #self.updateChannelSubscriptions()

        # invalidate the packae list cache
        # this forces up2date ot recaculate the available package list

        # FIXME: this is probabaly not valid anymore
        up2date.avail_package_list = []
        rhnChannel.channel_blacklist = []    
        blacklist_updated = None
        
        # tell it which channels to ignore
        for index in range(self.channelArea.n_rows):
            rowData = self.channelArea.get_row_data(index)
            if rowData[0] == 0:
                rhnChannel.channel_blacklist.append(rowData[1]['label'])
                # so we know to recreate the avail package lists. A little
                # wasteful, but correct. 
                blacklist_updated = TRUE

        if not self.gotPackages or blacklist_updated:
            self.pkgList = []
            self.selectedPkgList = []
            self.skipPkgList = []
            self.pkgsSelected = FALSE
            self.totalSelectedSize = long(0)
            self.druid.set_buttons_sensitive(TRUE, FALSE, TRUE)
            
            pwin = progress.Progress()
            pwin.setLabel(_("Red Hat Update Agent is building a list of updated\nRPM packages installed on your system.  Please wait."))

            try:
                up2dateAuth.maybeUpdateVersion()
                # override whatever this value is so we get good
                # interactive GUI response
                cfg.writeEntry("headerFetchCount", 20)
                # so we can get self.Plist.installedPackageHash()
                self.pList = packageList.PackageList()
                self.pList.msgCallback = pwin.setLabel
                self.pList.progressCallback = pwin.setProgress
                self.pList.ignoreMsgCallback = None
                self.pList.addForcePackages([])
                self.pList.run()
                self.pkgList = self.pList.getPackagesToInstall()
                self.skipPkgList = self.pList.getSkippedPackages()

            except up2dateErrors.RpmError, e:
                pwin.destroy()
                self.fatalError(_("There was a fatal RPM error.  The message was:\n") + e.errmsg)
            except up2dateErrors.FileNotFoundError, e:
                pwin.destroy()
                self.fatalError(_("There was a fatal error communicating with the server.  The message was:\n") + e.errmsg)
            except up2dateErrors.CommunicationError, e:
                pwin.destroy()
                self.fatalError(_("There was a fatal error communicating with the server.  The message was:\n") + e.errmsg)
            except OSError, e:
                self.fatalError(_("An unexpected OS error occurred:\n%s") % e)

            pwin.destroy()
            while gtk.events_pending():
                gtk.mainiteration(FALSE)

            self.gotPackages = TRUE

        if len(self.skipPkgList) > 0:
            self.druid.set_page(self.xml.get_widget("skippedPage"))
        else:
            self.druid.set_page(self.xml.get_widget("packagePage"))
            
        return TRUE

    def onSkippedPagePrepare(self, page, dummy):
        if not self.gotSkippedPackages:
            self.skippedArea.freeze()
            for pkg,reason in self.skipPkgList:
                try:
                    self.skippedArea.append_row((pkg[0], pkg[1],
                                                 pkg[2],
                                                 pkg[4],
                                                 "%d kB" % (int(pkg[5]) / 1024),
                                                 reason), FALSE, pkg)
                except up2dateErrors.RpmError, e:
                    dlg = gnome.ui.GnomeWarningDialog(e.errmsg, self.mainWin)
                    
            # get max num of chars and assuming a char size of roughly 8 pixels
            maxlength = max(map(lambda x: len(x[0][0]), self.skipPkgList)) * 8
            # smallest column width=100, largest=200, but will try to fit to
            # length of largest packagename.
            self.skippedArea.set_column_min_width(1, max(min(maxlength, 200), 100))
            self.skippedArea.thaw()

            self.gotSkippedPackages = TRUE

    def onSkippedPageNext(self, page, dummy):
        pass

    def onChannelsPagePrepare(self,page, dummy):
        
        descriptionLabel = self.xml.get_widget("SubChannelDescriptionLabel")

        if self.channelArea.n_rows:
            return
        
        channels = rhnChannel.getChannelDetails()

        self.channelArea.freeze()
        row = 0
        for channel in channels:
            values = []
            values.append(channel['name'])
            #values.append(channel['parent_channel'])
            values.append(channel['label'])
            values.append(channel['arch'])
            values.append(channel['description'])
            # kluge to store the initial status of the channel so
            # we know what channels to sub/unsub
            channel['initial_status'] = TRUE
            textlist =  tuple(values[:4])
            self.channelArea.append_row(textlist, TRUE, channel)
#            self.channelArea.set_row_data(row, (not not TRUE, channel))
#            row = row + 1
#            self.channelArea.n_rows = self.channelArea.n_rows + 1

            
                                        
                                          
        if channels:
            self.channelArea.select_row(0,0)
            
            
        self.channelArea.thaw()
        
    
    def onPackagePagePrepare(self, page, dummy):
        
        self.packageArea.freeze()
        self.packageArea.clear()
        self.totalSelectedSize = long(0)

        #FIXME: we need a hash of the installed package names -> the latest version installed
        #       we will use this to populate the "old version" info in the gui
        #       it might also be useful if we had a data structure clearly indicating what
        #       old package was replaced by what new package (including obsoletes...) that
        #       probabaly the best, since it may be useful to report that exact info at some point
        if len(self.skipPkgList) > 0:
            for index in range(self.skippedArea.n_rows):
                rowData = self.skippedArea.get_row_data(index)
                if rowData[0] == 1:
                    pkg = rowData[1]
                    self.packageArea.append_row((pkg[0],
                                                 pkg[1],
                                                 pkg[2],
                                                 pkg[4],
                                                 "%d kB" % (int(pkg[5]) / 1024)), TRUE, pkg)
                    try:
                        if not up2date.isPackageCached(pkg):
                            self.totalSelectedSize = self.totalSelectedSize + int(pkg[5])
                        self.pkgsSelected = TRUE
                    except up2dateErrors.RpmError, e:
                        self.fatalError(_("There was a fatal RPM error. The message was:\n") + e.errmsg)
        elif len(self.pkgList) == 0:
            dlg = gnome.ui.GnomeOkDialog(_("Your system is fully updated.  No new packages are needed."), self.mainWin)
            dlg.run_and_close()
            sys.exit(0)
            
        for pkg in self.pkgList:
            try:
                self.packageArea.append_row((pkg[0], pkg[1], pkg[2], pkg[4],
                                             "%d kB" % (int(pkg[5]) / 1024)), FALSE, pkg)
            except up2dateErrors.RpmError, e:
                dlg = gnome.ui.GnomeWarningDialog(e.errmsg, self.mainWin)

        # get max num of chars and assuming a char size of roughly 8 pixels.
        if len(self.pkgList):
            maxlength = max(map(lambda x: len(x[0]), self.pkgList)) * 8
        else:
            # 150 is just halfway between the min of 100 pixels and the max of 200 
            maxlength = 150
            
        # smallest column width=100, largest=200, but will try to fit to length
        # of largest packagename.
        self.packageArea.set_column_min_width(1, max(min(maxlength, 200), 100))
        self.xml.get_widget("sizeLabel").set_text("%d kB" % (self.totalSelectedSize / 1024))
        self.packageArea.thaw()
        

    def onPackagePageNext(self, page, dummy):
        def cb(foo):
            return foo

        if self.totalSelectedSize > up2dateUtils.freeDiskSpace():
            dlg = gnome.ui.GnomeErrorDialog(_("""The total size of selected packages is %d kB, but
You only have %d kB of free space.
Please unselect some packages and continue.""") % (self.totalSelectedSize / 1024,
                                                   up2dateUtils.freeDiskSpace() / 1024),
                                            self.mainWin)
            return TRUE

        if not self.pkgsSelected:
            dlg = gnome.ui.GnomeErrorDialog(_("""You must select at least one package.
If you do not wish to update any packages,
press the "Cancel" button to exit Update Agent."""), self.mainWin)
            return TRUE

        # build list of selected packages
        rowData = ()
        self.selectedPkgList = []
        newPackages = []
        for index in range(self.packageArea.n_rows):
            rowData = self.packageArea.get_row_data(index)
            if rowData[0] == 1:
                self.selectedPkgList.append(rowData[1])

        pwin = progress.Progress()

        while gtk.events_pending():
            gtk.mainiteration(FALSE)
        try:
            (self.depPackages,self.conflicts) = up2date.dryRun(self.selectedPkgList,
                                                               pwin.setLabel,
                                                               pwin.setProgress,
                                                               self.__refreshCallback)
        except up2dateErrors.CommunicationError, e:
            pwin.destroy()
            self.fatalError(_("There was a fatal error communicating with the server.  The message was:\n") + e.errmsg)
        except up2dateErrors.RpmError, e:
            pwin.destroy()
            dlg = self.fatalError(
                _("There was an RPM error.  The message was:\n") + e.errmsg
                )
        except up2dateErrors.RpmInstallError, e:
            pwin.destroy()
            dlg = self.fatalError(_("There was an RPM install error. The message was:\n") + e.errmsg)
        except up2dateErrors.DependencyError, e:
            pwin.destroy()
            dlg = gnome.ui.GnomeWarningDialog(
                _("There was a package dependency problem.  The message was:\n") + \
                e.errmsg + \
                _("\nPlease modify your package selections and try again.")
                )
            return TRUE
        except up2dateErrors.DependencySenseError, e:
            pwin.destroy()
            errmsg = "There was a dependency sense error with the sense: %s" % e.sense
            dlg = gnome.ui.GnomeWarningDialog(wrap_text(errmsg), self.mainWin)
            return TRUE
        except up2dateErrors.ConflictError, e:
            pwin.destroy()
            dlg = gnome.ui.GnomeWarningDialog(wrap_text(e.errmsg), self.mainWin)
            return TRUE
        except up2dateErrors.FileNotFoundError, e:
            pwin.destroy()
            dlg = gnome.ui.GnomeWarningDialog(wrap_text(e.errmsg), self.mainWin)
            return TRUE
        
        

        
        pwin.destroy()
        while gtk.events_pending():
            gtk.mainiteration(FALSE)

        # er, handle conflicts?

        if len(self.depPackages) > 0:
            self.druid.set_page(self.xml.get_widget("dependencyPage"))
            return TRUE
        else:
            self.druid.set_page(self.xml.get_widget("retrievalPage"))
            return TRUE


    def onPackagePageBack(self, page, dummy):
        if len(self.skipPkgList) > 0:
            self.druid.set_page(self.xml.get_widget("skippedPage"))
        else:
            self.druid.set_page(self.xml.get_widget("startPage"))
        return TRUE


    def onDependencyPagePrepare(self, page, dummy):
        area1 = self.xml.get_widget("dependencyArea1")
        area1.clear()
        for pkg in self.depPackages:
            area1.append(pkg)
            # dont add to total size if we already have it downloaded
            if not up2date.isPackageCached(pkg):
                self.totalSelectedSize = self.totalSelectedSize + int(pkg[5])
            self.pkgsSelected = TRUE

        if self.totalSelectedSize > up2dateUtils.freeDiskSpace():
            dlg = gnome.ui.GnomeErrorDialog(_("""The total size of selected packages is %d kB, but
You only have %d kB of free space.
Please unselect some packages and continue.""") % (self.totalSelectedSize / 1024,
                                                   up2dateUtils.freeDiskSpace() / 1024),
                                            self.mainWin)
            self.druid.set_page(self.xml.get_widget("packagePage"))
	    return TRUE


    def setRetrievalProgress(self, amount, total, speed = 0, secs = 0):
        if total:
            i = float(amount) / total
        else:
            i = 1
            
        self.progress.update(i)

        if speed:
            trans_bps_str = "%d of %d kB transferred at %d kB/sec\n" % \
                            (amount/1024, total/1024, speed/1024)
            totalTime = total / speed
            trans_time_str = "Package transfer time: %02d:%02d:%02d" % \
                             (totalTime / 3600, (totalTime % 3600) / 60,
                              totalTime % 60)
            trans_rem_str = " (%02d:%02d:%02d remaining)" % \
                            (secs / 3600, (secs % 3600) / 60,
                             (secs % 60))
            tstr = trans_bps_str + trans_time_str + trans_rem_str
        else:
            tstr = _("Package already downloaded")

        self.xml.get_widget("timeLabel").set_text(tstr)
        
        while (gtk.events_pending()):
            gtk.mainiteration(FALSE)

    def startRetrievalTimer(self):
        pass

    def labelcb(self, name, shortName=""):
        label = self.xml.get_widget("retrievePackageNameLabel")
        label.set_text(name)
            

    def doRetrieval(self):

        def cb(arg):
            return arg

        gtk.threads_enter()
        self.progress = self.xml.get_widget("retrievePackageProgress")
        totalProgress = self.xml.get_widget("retrieveTotalProgress")
        totalProgress.update(0)
        

        self.setBusyCursor()
        
        pkgCount = len(self.selectedPkgList)
        totalRetrievedSize = 0

        # since we dont want to download any packages
        # just set the count to 0, so we dont have to
        # further nest this mess
        if cfg.readEntry("noDownload"):
            pkgCount = 0
            
        for index in range(pkgCount):
            self.startRetrievalTimer()
            hdr = up2date.getHeader(self.selectedPkgList[index],lite=1)
            self.xml.get_widget("retrieveInfoLabel").set_text("%s\n%s" %
                                                              (hdr['summary'],
                                                              hdr['description']))
            try:
                up2date.getPackage(self.selectedPkgList[index],
                                   self.labelcb,
                                   self.setRetrievalProgress)
            except up2dateErrors.CommunicationError, e:
                self.fatalError(_("Error while retrieving package %s.\nThe message was:\n%s") % (up2dateUtils.pkgToString(self.selectedPkgList[index]), e.errmsg))
            except up2dateErrors.FileNotFoundError, e:
                self.fatalError(_("There was a fatal error communicating with the server.  The message was:\n") + e.errmsg)
            
            # do signature checking only if GPG installation checks out,
            # and GPG checking is enabled.
            # we already warned them about any errors when we started up.
            if cfg.readEntry("useGPG") and not gpgUtils.checkGPGInstallation():
                try:
                    res = up2date.hasBadSignature(self.selectedPkgList[index])
                except up2dateErrors.RpmError, e:
                    self.fatalError(_("A fatal RPM error has occurred.  The message was:\n%s") % e.errmsg)
                if res == 1:
                    dlg = gnome.ui.GnomeQuestionDialog(_("The package %s is not signed with a GPG signature.  Continue?") % up2dateUtils.pkgToString(self.selectedPkgList[index]), cb)
                    if dlg.run_and_close():
                        sys.exit(1)
                if res == 2:
                    dlg = gnome.ui.GnomeQuestionDialog(_("The package %s does not have a valid GPG signature.\nIt has been tampered with or corrupted.  Continue?") % up2dateUtils.pkgToString(self.selectedPkgList[index]), cb)
                    if dlg.run_and_close():
                        sys.exit(1)

            totalRetrievedSize = totalRetrievedSize + \
                                 int(self.selectedPkgList[index][5])

            if self.totalSelectedSize:
                i = float(totalRetrievedSize) / self.totalSelectedSize
            else:
                i = 1
            totalProgress.update(i)
            while gtk.events_pending():
                gtk.mainiteration(FALSE)

        self.xml.get_widget("packagesFinishedLabel").set_text(_("All finished.  Click \"Next\" to continue."))
        self.druid.set_buttons_sensitive(TRUE, TRUE, TRUE)
        # XXX remove me when locking is fixed in gtk
        self.setArrowCursor()
        gtk.threads_leave()
        return 0


    def onRetrievalPagePrepare(self, page, dummy):
        self.druid.set_buttons_sensitive(FALSE, FALSE, TRUE)
        while gtk.events_pending():
            gtk.mainiteration(FALSE)

        # total hack to get this function to start as soon as this
        # on exits.  GTK makes me do ugly things.
        gtk.timeout_add(0, self.doRetrieval)


    def onRetrievalPageNext(self, page, dummy):
        if cfg.readEntry("retrieveOnly"):
            self.druid.set_page(self.xml.get_widget("finishPage"))
        else:
            self.druid.set_page(self.xml.get_widget("installPage"))
        return TRUE

    def onRetrievalPageBack(self, page, dummy):
        self.xml.get_widget("packagesFinishedLabel").set_text("...")
        self.druid.set_page(self.xml.get_widget("packagePage"))
        return TRUE
    

    def rpmCallback(self, what, amount, total, hdr, path):
        if what == rpm.RPMCALLBACK_INST_OPEN_FILE:
            fileName = "%s/%s-%s-%s.%s.rpm" % (path,
                                               hdr['name'],
                                               hdr['version'],
                                               hdr['release'],
                                               hdr['arch'])
            try:
                self.fd = os.open(fileName, os.O_RDONLY)
            except:
                raise up2dateErrors.RpmError(_("Error opening %s") % fileName)
            
            return self.fd
        elif what == rpm.RPMCALLBACK_INST_START:
            fileName = "%s/%s-%s-%s.%s.rpm" % (path,
                                               hdr['name'],
                                               hdr['version'],
                                               hdr['release'],
                                               hdr['arch'])
            self.pkgIndex = self.pkgIndex + 1
            self.pkgLabel.set_text(fileName)
        elif what == rpm.RPMCALLBACK_INST_CLOSE_FILE:
            os.close(self.fd)
            # we used to do a up2date.remoteAddPackage() here
            
            i = (self.pkgIndex * 1.0) / self.pkgCount
            if i > 1:
                i = 1.0
            elif i < 0:
                i = 0
            self.totalProgress.update(i)

        elif what == rpm.RPMCALLBACK_INST_PROGRESS:
	    if total == 0:
		i = 0
	    else: 
            	i = (amount + 1.0) / total
            if i > 1:
                i = 1.0
            elif i < 0:
                i = 0
            self.progress.update(i)
        elif what == rpm.RPMCALLBACK_UNINST_STOP:
            # we used to do a up2date.remoteDelPackage() here
            pass
        
        if hasattr(rpm, "RPMCALLBACK_UNPACK_ERROR"):
            if ((what == rpm.RPMCALLBACK_UNPACK_ERROR) or
                (what == rpm.RPMCALLBACK_CPIO_ERROR)):
                pkg = "%s-%s-%s" % (hdr[rpm.RPMTAG_NAME],
                                    hdr[rpm.RPMTAG_VERSION],
                                    hdr[rpm.RPMTAG_RELEASE])
                
                if what == rpm.RPMCALLBACK_UNPACK_ERROR:
                    raise up2dateErrors.RpmInstallError, ("There was a rpm unpack error  installing the package: %s" % pkg, pkg)
                elif what == rpm.RPMCALLBACK_CPIO_ERROR:
                    raise up2dateErrors.RpmInstallError, ("There was a cpio error installing the package: %s" % pkg, pkg)

        
        while gtk.events_pending():
            gtk.mainiteration(FALSE)


    def doInstallation(self):
        gtk.threads_enter()
        self.progress = self.xml.get_widget("installPackageProgress")
        self.totalProgress = self.xml.get_widget("installTotalProgress")
        self.totalProgress.update(0)
        self.pkgLabel = self.xml.get_widget("installPackageNameLabel")
        self.pkgIndex = 0
        self.pkgCount = len(self.selectedPkgList)

        self.setBusyCursor()
        try:
            kernelsToInstall = up2date.installPackages(self.selectedPkgList, self.rpmCallback)
        except up2dateErrors.CommunicationError, e:
            self.setArrowCursor()
            self.fatalError(_("There was a fatal RPM error. The message was:\n") + e.errmsg)
        except up2dateErrors.RpmError, e:
            self.setArrowCursor()
            self.fatalError(_("There was a fatal RPM error.  The message was:\n") + e.errmsg)
        except up2dateErrors.RpmInstallError, e:
            self.setArrowCursor()
            self.fatalError(_("There was a fatal RPM install error. The message was:\n") + e.errmsg)

        
        if len(kernelsToInstall) and lilocfg:
            try:
                up2date.installBootLoader(kernelsToInstall)
            except lilocfg.LiloConfError, e:
                dlg = gnome.ui.GnomeWarningDialog(e.errmsg, self.mainWin)
            except lilocfg.LiloInstallError, e:
                dlg = gnome.ui.GnomeWarningDialog(e.errmsg, self.mainWin)
                
        if not cfg.readEntry("keepAfterInstall"):
            for pkg in self.selectedPkgList:
                try:
                    up2date.removePackage(pkg)
                except up2dateErrors.FileError, e:
                    dlg = gnome.ui.GnomeWarningDialog(e.errmsg, self.mainWin)

                while gtk.events_pending():
                    gtk.mainiteration(FALSE)

        self.xml.get_widget("installingFinishedLabel").set_text(_("All finished.  Click \"Next\" to continue."))
        self.druid.set_buttons_sensitive(FALSE, TRUE, TRUE)
        # XXX remove me when locking is fixed in gtk
        self.setArrowCursor()
        gtk.threads_leave()
        return 0
    

    def onInstallPagePrepare(self, page, dummy):
        self.druid.set_buttons_sensitive(FALSE, FALSE, TRUE)
        while gtk.events_pending():
            gtk.mainiteration(FALSE)

        # total hack to get this function to start as soon as this
        # on exits.  GTK makes me do ugly things.
        gtk.timeout_add(0, self.doInstallation)


    def onInstallPageBack(self, page, dummy):
        self.xml.get_widget("installingFinishedLabel").set_text("...")
        self.druid.set_page(self.xml.get_widget("packagePage"))
        return TRUE


    def onFinishPagePrepare(self, page, dummy):
        druid = self.xml.get_widget("druid")
        druid.set_buttons_sensitive(FALSE, TRUE, FALSE)
        druid.set_show_finish(TRUE)
        
        text = _("The Red Hat Update Agent has finished ")
        if cfg.readEntry("retrieveOnly"):
            text = text + _("retrieving\n the following packages successfully:\n\n")
        else:
            text = text + _("installing\n the following packages successfully:\n\n")

        for pkg in self.selectedPkgList:
            text = text + up2dateUtils.pkgToString(pkg) + "\n"

        if cfg.readEntry("retrieveOnly"):
            text = text + _("""
Because of your settings, these packages were only downloaded,
not installed.  If you wish to change your settings, please run the
"up2date-config" program.""")

        self.xml.get_widget("finishArea").insert_defaults(text)
        return FALSE


    def onFinishPageFinish(self, page, dummy=None):
        gtk.mainquit()
    

def showAbout():
    dlg = gnome.ui.GnomeAbout(_("About Red Hat Update Agent"),
                              up2date.version(),
                              "Copyright (c) 1999-2001 Red Hat, Inc.",
                              ["Preston Brown <pbrown@redhat.com>",
                               "Adrian Likins <alikins@redhat.com>",
                               "Cristian Gafton <gafton@redhat.com>"],
                              _("This software is distributed under the GPL.  Please Report bugs to Red Hat's Bug Tracking System: http://bugzilla.redhat.com/bugzilla/"))
    dlg.run_and_close()

def rootWarning():
    dlg = gnome.ui.GnomeErrorDialog(_("You must run the Update Agent as root."))
    dlg.run_and_close()

def errorWindow(msg):
    dlg = gnome.ui.GnomeErrorDialog(wrap_text(msg))
    dlg.run_and_close()

def main():
    def cb(foo):
        return foo
        
    test = 0

    if len(saveargs) > 0:
        if saveargs[0] == "-t" or saveargs[0] == "--test":
            test = 1
        
    signal.signal(signal.SIGINT, signal.SIG_DFL)

    if os.geteuid() != 0 and not test:
        rootWarning()
        sys.exit(1)

    try:
        from gtk import _disable_gdk_threading
        _disable_gdk_threading()
    except ImportError:
        pass

    gui = Gui()
    gtk.mainloop()

    
if __name__ == "__main__":
    try:
        from rhn import rpclib
    except ImportError:
        rpclib = __import__("xmlrpclib")
        
    try:
        main()
    except rpclib.ResponseError, e:
        print e
    except IOError, e:
        print _("There was some sort of I/O error: %s") % e.errmsg
